"use strict";
this.name        = "miningcontracts";
this.author      = "Diagoras";
this.copyright   = "2015 Maxim Savchenko";
this.licence     = "Ms-PL";
this.description = "Excavate asteroids for reward with deadline conditions.";

this.startUp = function() {
	this.MC_contracts = [];

	if (missionVariables.MC_initialized)
	{
		this.MC_system = missionVariables.MC_system;
		this.MC_clock = missionVariables.MC_clock;
		var s = missionVariables.MC_contracts;
		if (s && s.length > 0) this.MC_contracts = JSON.parse(s);
	}
	else
	{
		this.MC_system = "";
		this.MC_clock = 0;
	}
};

this.startUpComplete = function() {
	this.MC_setInstructions();
	if(player.ship.docked) this.shipDockedWithStation(player.ship.dockedStation);
};

this.shipDockedWithStation = function(station)
{
	if (station.shipClassName != "Rock Hermit") return;

	station.setInterface("MiningContracts-bm", {
		title: "Mining Contracts",
		category: "Contracts",
		summary: "Sign up for a local mining operations.",
		callback: this.MC_Interface.bind(this)
	});
};

this.playerWillSaveGame = function(message)
{
	missionVariables.MC_initialized = 1;
	missionVariables.MC_system = this.MC_system;
	missionVariables.MC_clock = this.MC_clock;
	missionVariables.MC_contracts = JSON.stringify(this.MC_contracts);
};

this.shipExitedWitchspace = function()
{
	var number = this.MC_contracts.length;
	var total = 0;

	for (var i = 0; i < number; i++)
	{
		var contract = this.MC_contracts[i];
		if (contract.signed) total += contract.amount * contract.penalty - contract.reward;
	}

	this.MC_contracts = [];

	if (total > 0)
	{
		player.bounty += 50;
		player.commsMessage(
			"Mining Association of " +
			this.MC_system +
			" have offered bounty of " +
			formatCredits(50, true, true) +
			" for your head");
	}

	this.MC_setInstructions();
};

this.MC_setInstructions = function()
{
	var total = {
		alloys: 0,
		minerals: 0
	};

	var number = this.MC_contracts.length;

	for (var i = 0; i < number; i++)
	{
		var contract = this.MC_contracts[i];
		if (contract.signed == 1) total[contract.cargo] += contract.amount;
	}

	var alloys = total.alloys > 0;
	var minerals = total.minerals > 0;

	if (alloys || minerals)
	{
		this.MC_active = true;
		var alloysMsg = alloys ? total.alloys + "t of alloys" : "";
		var mineralsMsg = minerals ? total.minerals + "t of minerals" : "";
		var andMsg = alloys && minerals ? " and " : "";
		mission.setInstructions("Mining Contracts: " + alloysMsg + andMsg + mineralsMsg);
	}
	else
	{
		this.MC_active = false;
		mission.setInstructions(null);
	}
};

this.MC_clockStringToMinutes = function(time)
{
	return clock.clockStringForTime(time).slice(4,13);
};

this.MC_Interface = function()
{
	if (this.MC_system != system.name)
	{
		this.MC_system = system.name;
		this.MC_clock = 0;
		this.MC_contracts = [];
	}

	this.MC_generateContracts();
	this.MC_expireContracts();

	var number = this.MC_contracts.length;

	var parameters = {
		title: "Mining Contracts",
		message: "Sign up for a local mining operations.",
		choices: {},
		allowInterrupt: true
	};

	for (var i = 0; i < number; i++)
	{
		var contract = this.MC_contracts[i];
		var status = ["OFFER", "TAKEN", "BLOWN"][contract.signed];
		parameters.choices["C" + i]
			= status + ": "
			+ contract.amount + "t of "
			+ contract.cargo + " before "
			+ this.MC_clockStringToMinutes(contract.deadline) + " for "
			+ formatCredits(contract.reward, true, true);
	}

	mission.runScreen(parameters, this.MC_contractDetails.bind(this));
};

this.MC_contractDetails = function(choice)
{
	if (choice == null) return;
	var cid = choice.replace("C", "");
	var contract = this.MC_contracts[cid];
	var header, descr, ps, id, option;

	if (contract.signed == 0)
	{
		header = "PUBLIC CONTRACTUAL OFFER FOR MINING OPERATIONS";
		descr = "Subject of delivery";
		ps = 
			"Terms and conditions:\n" +
			"Cargo must be delivered before due date sharp. " +
			"If forefeit penalty exceeds fee, the difference becomes contractor's debt liability. " +
			"Contractor must not leave system before completion of obligations or debt retirement." +
			"\n\nn.b. Mining Association of " + this.MC_system +
			" reserves the right to set a bounty on heads of defaulting contractors.";
		id = "SIGN";
		option = "Sign contract";
	}
	else if (contract.signed == 2)
	{
		header = "TERMINATED CONTRACT";
		descr = "Deficiency";
		var balance = contract.reward - contract.amount * contract.penalty;
		if (balance < 0)
		{
			ps = 
				"You have failed to perform a contract and forefeit penalty exceeds your fee. " +
				"According to the terms your debt to " + contract.company + " is " + formatCredits(-balance, true, true) + ". " +
				"If you leave system before repayment, company will report your default to Mining Association. " +
				"Please, do not delay your payment." +
				"\n\nn.b. Mining Association of " + this.MC_system +
				" reserves the right to set a bounty on heads of defaulting contractors.";
			id = "DEBT";
			option = "Pay debt";
		}
		else
		{
			ps = 
				"You have failed to perform a contract fully and forefeit penalty have been deducted from your fee. " +
				"According to the terms your reward is " + formatCredits(balance, true, true) + ". " +
				"Please, be more diligent next time.";
			id = "REWARD";
			option = "Collect reward";
		}
	}
	else
	{
		descr = "Deficiency";
		if (contract.amount > 0)
		{
			header = "OPERATING CONTRACT";
			ps = 
				"Terms and conditions:\n" +
				"Cargo must be delivered before due date sharp. " +
				"If forefeit penalty exceeds fee, the difference becomes contractor's debt liability. " +
				"Contractor must not leave system before completion of obligations or debt retirement." +
				"\n\nn.b. Mining Association of " + this.MC_system +
				" reserves the right to set a bounty on heads of defaulting contractors.";
			id = "CARGO";
			option = "Dispatch cargo";
		}
		else
		{
			header = "CONTRACT COMPLETION";
			ps = 
				"You have successfully performed a contract. " +
				"According to the terms your reward is " + formatCredits(contract.reward, true, true) + ". " +
				"We are looking forward to working with you in the future.";
			id = "REWARD";
			option = "Collect reward";
		}
	}

	var parameters = {
		title: "Mining Contracts",
		message:
			header + "\n" +
			"\n" +
			"Ordering party: " + contract.company + "\n" +
			descr + ": " + contract.amount + "t of " + contract.cargo + "\n" +
			"Due date: " + clock.clockStringForTime(contract.deadline) + "\n" +
			"Fee: " + formatCredits(contract.reward, true, true) + "\n" +
			"Forfeit penalty: " + formatCredits(contract.penalty, true, true) + " per ton of shortage\n" +
			"\n" +
			ps,
		choices: {},
		allowInterrupt: id != "REWARD"
	};

	parameters.choices["0_" + id + ":" + cid] = option;

	if (parameters.allowInterrupt)
	{
		if (id == "CARGO") parameters.choices["1_TERMINATE:" + cid] = "Terminate contract";
		parameters.choices["2_BACK"] = "Back to the list";
		if (id == "SIGN") parameters.initialChoicesKey = "2_BACK";
	}

	mission.runScreen(parameters, this.MC_contractAction.bind(this));
};

this.MC_contractAction = function(choice)
{
	if (choice == null) return;
	var ch = choice.split(":");

	switch (ch[0])
	{
		case "0_SIGN":
			var idx = ch[1];
			this.MC_contracts[idx].signed = 1;
			player.consoleMessage("CONTRACT SIGNED");
			this.MC_setInstructions();
			this.MC_contractDetails("C" + idx);
			break;

		case "0_DEBT":
			var idx = ch[1];
			var contract = this.MC_contracts[idx];
			var debt = contract.amount * contract.penalty - contract.reward;

			if (debt > player.credits)
			{
				player.consoleMessage("NOT ENOUGH CREDITS");
				this.MC_contractDetails("C" + idx);
			}
			else
			{
				player.commsMessage(formatCredits(debt, true, true) + " deducted");
				player.credits -= debt;
				this.MC_contracts.splice(idx, 1);
				this.MC_Interface();
			}

			break;

		case "0_REWARD":
			var idx = ch[1];
			var contract = this.MC_contracts[idx];
			var reward = contract.reward - contract.amount * contract.penalty;
			player.commsMessage(formatCredits(reward, true, true) + " received");
			player.credits += reward;
			this.MC_contracts.splice(idx, 1);
			this.MC_Interface();
			break;

		case "0_CARGO":
			var idx = ch[1];
			var contract = this.MC_contracts[idx];
			var amount = Math.min(contract.amount, manifest[contract.cargo]);

			if (amount > 0)
			{
				player.commsMessage(amount + "t of " + contract.cargo + " dispatched");
				this.MC_contracts[idx].amount -= amount;
				manifest[contract.cargo] -= amount;
				this.MC_setInstructions();
			}
			else player.consoleMessage("no " + contract.cargo + " on board");

			this.MC_contractDetails("C" + idx);
			break;

		case "1_TERMINATE":
			var idx = ch[1];
			var contract = this.MC_contracts[idx];

			var parameters = {
				title: "Mining Contracts",
				message: "Are you sure you want to prematurely terminate contract with " + contract.company + "?",
				choices: {},
				allowInterrupt: true
			};

			parameters.choices["0_TERMYES:" + idx] = "Yes";
			parameters.choices["1_TERMNO:" + idx] = "No";
			parameters.initialChoicesKey = "1_TERMNO:" + idx;
			mission.runScreen(parameters, this.MC_contractAction.bind(this));
			break;

		case "0_TERMYES":
			var idx = ch[1];
			this.MC_contracts[idx].signed = 2;
			player.consoleMessage("CONTRACT TERMINATED");
			this.MC_setInstructions();
			this.MC_contractDetails("C" + idx);
			break;

		case "1_TERMNO":
			var idx = ch[1];
			this.MC_contractDetails("C" + idx);
			break;

		case "2_BACK":
			this.MC_Interface();
			break;
	}
};

this.MC_randomElement = function(array)
{
	return array[Math.floor(Math.random() * array.length)];
};

this.MC_randomExponential = function()
{
	return -Math.log(1 - Math.random());
};

this.MC_randomNormals = function()
{
	var u = Math.sqrt(this.MC_randomExponential());
	var v = 2 * Math.PI * Math.random();

	return {
		x: u * Math.cos(v),
		y: u * Math.sin(v)
	};
};

this.MC_ceilScale = function(value, scale)
{
	return scale * Math.ceil(value / scale);
};

this.MC_companyPatterns = [
	"? Company Limited",
	"? plc",
	"? S.A.",
	"? Group",
	"? Gold Corporation",
	"? Inc.",
	"Potash Corporation of ? Inc.",
	"? Copper Corporation",
	"MMC ? Nickel",
	"? Platinum Limited",
	"? Mining Corporation",
	"Aluminum Corporation of ? Limited",
	"? Platinum Holdings Limited",
	"? Mining Limited",
	"?Gold",
	"? Goldfields Corporation",
	"National Copper Corporation of ?",
	"? Industries Ltd."
];

this.MC_companyName = function()
{
	return this.MC_randomElement(this.MC_companyPatterns).replace("?", randomName());
};

this.MC_randomCargo = function()
{
	return this.MC_randomElement(["alloys", "minerals"]);
	//if(worldScripts["Spawn-staer9_asteroid"]) return this.MC_randomElement(["alloys", "minerals"]);
	//else return "minerals";
};

this.MC_logistic = function(x)
{
	return 1 / (1 + Math.exp(x));
};

this.MC_createContract = function(time)
{
	var rn = this.MC_randomNormals();
	var amount = Math.ceil(Math.exp(rn.x + 3));
	var deadline = this.MC_ceilScale(time + amount * Math.exp(rn.y / 2 + 6.5), 900);
	var expires = (time + deadline) / 2;
	var cargo = this.MC_randomCargo();
	var price = player.ship.dockedStation.market[cargo].price / 10;
	var reward = this.MC_ceilScale(amount * price * this.MC_logistic(rn.y), 10);

	var contract = {
		company: this.MC_companyName(),
		expires: expires,
		deadline: deadline,
		cargo: cargo,
		amount: amount,
		reward: reward,
		penalty: price,
		signed: 0
	};

	this.MC_contracts.push(contract);
};

this.MC_generateContracts = function()
{
	var prevClock = this.MC_clock;
	var nextClock = clock.seconds;
	var pregen = 96 * 3600;
	if (nextClock - prevClock > pregen) prevClock = nextClock - pregen;

	while (true)
	{
		prevClock += this.MC_randomExponential() * 1800;
		if (prevClock > nextClock) break;
		this.MC_createContract(prevClock);
	};

	this.MC_clock = nextClock;
};

this.MC_expireContracts = function()
{
	var number = this.MC_contracts.length;
	var updated = [];

	for (var i = 0; i < number; i++)
	{
		var contract = this.MC_contracts[i];
		if (contract.signed == 1 && contract.deadline < clock.seconds) contract.signed = 2;
		if (contract.signed || contract.expires > this.MC_clock) updated.push(contract);
	}

	this.MC_contracts = updated;
	this.MC_setInstructions();
};